@@ -166,8 +166,8 @@ impl EnvVars {
     pub const UV_COMPILE_BYTECODE_TIMEOUT: &'static str = "UV_COMPILE_BYTECODE_TIMEOUT";
 
     /// Equivalent to the `--no-editable` command-line argument. If set, uv
-    /// installs any editable dependencies, including the project and any workspace members, as
-    /// non-editable
+    /// installs or exports any editable dependencies, including the project and any workspace
+    /// members, as non-editable.
     pub const UV_NO_EDITABLE: &'static str = "UV_NO_EDITABLE";
 
     /// Equivalent to the `--no-binary` command-line argument. If set, uv will install
@@ -225,6 +225,9 @@ impl EnvVars {
     /// Equivalent to the `--preview` argument. Enables preview mode.
     pub const UV_PREVIEW: &'static str = "UV_PREVIEW";
 
+    /// Equivalent to the `--preview-features` argument. Enables specific preview features.
+    pub const UV_PREVIEW_FEATURES: &'static str = "UV_PREVIEW_FEATURES";
+
     /// Equivalent to the `--token` argument for self update. A GitHub token for authentication.
     pub const UV_GITHUB_TOKEN: &'static str = "UV_GITHUB_TOKEN";
 
@@ -235,6 +238,14 @@ impl EnvVars {
     /// Equivalent to the `--allow-insecure-host` argument.
     pub const UV_INSECURE_HOST: &'static str = "UV_INSECURE_HOST";
 
+    /// Disable ZIP validation for streamed wheels and ZIP-based source distributions.
+    ///
+    /// WARNING: Disabling ZIP validation can expose your system to security risks by bypassing
+    /// integrity checks and allowing uv to install potentially malicious ZIP files. If uv rejects
+    /// a ZIP file due to failing validation, it is likely that the file is malformed; consider
+    /// filing an issue with the package maintainer.
+    pub const UV_INSECURE_NO_ZIP_VALIDATION: &'static str = "UV_INSECURE_NO_ZIP_VALIDATION";
+
     /// Sets the maximum number of in-flight concurrent downloads that uv will
     /// perform at any given time.
     pub const UV_CONCURRENT_DOWNLOADS: &'static str = "UV_CONCURRENT_DOWNLOADS";
@@ -728,9 +739,15 @@ impl EnvVars {
     /// the installer from modifying shell profiles or environment variables.
     pub const UV_UNMANAGED_INSTALL: &'static str = "UV_UNMANAGED_INSTALL";
 
+    /// The URL from which to download uv using the standalone installer. By default, installs from
+    /// uv's GitHub Releases. `INSTALLER_DOWNLOAD_URL` is also supported as an alias, for backwards
+    /// compatibility.
+    pub const UV_DOWNLOAD_URL: &'static str = "UV_DOWNLOAD_URL";
+
     /// Avoid modifying the `PATH` environment variable when installing uv using the standalone
-    /// installer and `self update` feature.
-    pub const INSTALLER_NO_MODIFY_PATH: &'static str = "INSTALLER_NO_MODIFY_PATH";
+    /// installer and `self update` feature. `INSTALLER_NO_MODIFY_PATH` is also supported as an
+    /// alias, for backwards compatibility.
+    pub const UV_NO_MODIFY_PATH: &'static str = "UV_NO_MODIFY_PATH";
 
     /// Skip writing `uv` installer metadata files (e.g., `INSTALLER`, `REQUESTED`, and `direct_url.json`) to site-packages `.dist-info` directories.
     pub const UV_NO_INSTALLER_METADATA: &'static str = "UV_NO_INSTALLER_METADATA";
