@@ -17,6 +17,7 @@ use uv_configuration::{
     Preview, PreviewFeatures, TargetTriple, Upgrade,
 };
 use uv_dispatch::BuildDispatch;
+use uv_distribution::LoweredExtraBuildDependencies;
 use uv_distribution_types::{
     DirectorySourceDist, Dist, Index, Requirement, Resolution, ResolvedDist, SourceDist,
 };
@@ -227,7 +228,7 @@ pub(crate) async fn sync(
             // Parse the requirements from the script.
             let spec = script_specification(script.into(), &settings.resolver)?.unwrap_or_default();
             let script_extra_build_requires =
-                script_extra_build_requires(script.into(), &settings.resolver)?;
+                script_extra_build_requires(script.into(), &settings.resolver)?.into_inner();
 
             // Parse the build constraints from the script.
             let build_constraints = script
@@ -248,7 +249,7 @@ pub(crate) async fn sync(
                 });
 
             match update_environment(
-                Deref::deref(&environment).clone(),
+                environment.clone(),
                 spec,
                 modifications,
                 build_constraints.unwrap_or_default(),
@@ -585,6 +586,7 @@ pub(super) async fn do_sync(
         no_build_isolation,
         no_build_isolation_package,
         extra_build_dependencies,
+        extra_build_variables,
         exclude_newer,
         link_mode,
         compile_bytecode,
@@ -602,12 +604,12 @@ pub(super) async fn do_sync(
         );
     }
 
-    // Lower the extra build dependencies with source resolution
+    // Lower the extra build dependencies with source resolution.
     let extra_build_requires = match &target {
         InstallTarget::Workspace { workspace, .. }
         | InstallTarget::Project { workspace, .. }
         | InstallTarget::NonProjectWorkspace { workspace, .. } => {
-            uv_distribution::ExtraBuildRequires::from_workspace(
+            LoweredExtraBuildDependencies::from_workspace(
                 extra_build_dependencies.clone(),
                 workspace,
                 index_locations,
@@ -630,6 +632,7 @@ pub(super) async fn do_sync(
                 no_build_isolation,
                 no_build_isolation_package: no_build_isolation_package.to_vec(),
                 extra_build_dependencies: extra_build_dependencies.clone(),
+                extra_build_variables: extra_build_variables.clone(),
                 prerelease: PrereleaseMode::default(),
                 resolution: ResolutionMode::default(),
                 sources,
@@ -637,7 +640,8 @@ pub(super) async fn do_sync(
             };
             script_extra_build_requires((*script).into(), &resolver_settings)?
         }
-    };
+    }
+    .into_inner();
 
     let client_builder = BaseClientBuilder::new()
         .retries_from_env()?
@@ -711,6 +715,9 @@ pub(super) async fn do_sync(
     // If necessary, convert editable to non-editable distributions.
     let resolution = apply_editable_mode(resolution, editable);
 
+    // Constrain any build requirements marked as `match-runtime = true`.
+    let extra_build_requires = extra_build_requires.match_runtime(&resolution)?;
+
     index_locations.cache_index_credentials();
 
     // Populate credentials from the target.
@@ -757,7 +764,7 @@ pub(super) async fn do_sync(
     let build_dispatch = BuildDispatch::new(
         &client,
         cache,
-        build_constraints,
+        &build_constraints,
         venv.interpreter(),
         index_locations,
         &flat_index,
@@ -768,6 +775,7 @@ pub(super) async fn do_sync(
         config_settings_package,
         build_isolation,
         &extra_build_requires,
+        extra_build_variables,
         link_mode,
         build_options,
         &build_hasher,
@@ -789,9 +797,6 @@ pub(super) async fn do_sync(
         build_options,
         link_mode,
         compile_bytecode,
-        index_locations,
-        config_setting,
-        config_settings_package,
         &hasher,
         &tags,
         &client,
@@ -947,8 +952,8 @@ impl From<&VirtualProject> for ProjectReport {
 impl From<&SyncTarget> for TargetName {
     fn from(target: &SyncTarget) -> Self {
         match target {
-            SyncTarget::Project(_) => TargetName::Project,
-            SyncTarget::Script(_) => TargetName::Script,
+            SyncTarget::Project(_) => Self::Project,
+            SyncTarget::Script(_) => Self::Script,
         }
     }
 }
@@ -1014,8 +1019,8 @@ enum TargetName {
 impl std::fmt::Display for TargetName {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
-            TargetName::Project => write!(f, "project"),
-            TargetName::Script => write!(f, "script"),
+            Self::Project => write!(f, "project"),
+            Self::Script => write!(f, "script"),
         }
     }
 }
@@ -1037,16 +1042,16 @@ enum SyncAction {
 impl From<&SyncEnvironment> for SyncAction {
     fn from(env: &SyncEnvironment) -> Self {
         match &env {
-            SyncEnvironment::Project(ProjectEnvironment::Existing(..)) => SyncAction::Check,
-            SyncEnvironment::Project(ProjectEnvironment::Created(..)) => SyncAction::Create,
-            SyncEnvironment::Project(ProjectEnvironment::WouldCreate(..)) => SyncAction::Create,
-            SyncEnvironment::Project(ProjectEnvironment::WouldReplace(..)) => SyncAction::Replace,
-            SyncEnvironment::Project(ProjectEnvironment::Replaced(..)) => SyncAction::Update,
-            SyncEnvironment::Script(ScriptEnvironment::Existing(..)) => SyncAction::Check,
-            SyncEnvironment::Script(ScriptEnvironment::Created(..)) => SyncAction::Create,
-            SyncEnvironment::Script(ScriptEnvironment::WouldCreate(..)) => SyncAction::Create,
-            SyncEnvironment::Script(ScriptEnvironment::WouldReplace(..)) => SyncAction::Replace,
-            SyncEnvironment::Script(ScriptEnvironment::Replaced(..)) => SyncAction::Update,
+            SyncEnvironment::Project(ProjectEnvironment::Existing(..)) => Self::Check,
+            SyncEnvironment::Project(ProjectEnvironment::Created(..)) => Self::Create,
+            SyncEnvironment::Project(ProjectEnvironment::WouldCreate(..)) => Self::Create,
+            SyncEnvironment::Project(ProjectEnvironment::WouldReplace(..)) => Self::Replace,
+            SyncEnvironment::Project(ProjectEnvironment::Replaced(..)) => Self::Update,
+            SyncEnvironment::Script(ScriptEnvironment::Existing(..)) => Self::Check,
+            SyncEnvironment::Script(ScriptEnvironment::Created(..)) => Self::Create,
+            SyncEnvironment::Script(ScriptEnvironment::WouldCreate(..)) => Self::Create,
+            SyncEnvironment::Script(ScriptEnvironment::WouldReplace(..)) => Self::Replace,
+            SyncEnvironment::Script(ScriptEnvironment::Replaced(..)) => Self::Update,
         }
     }
 }
@@ -1055,22 +1060,22 @@ impl SyncAction {
     fn message(&self, target: TargetName, dry_run: bool) -> Option<&'static str> {
         let message = if dry_run {
             match self {
-                SyncAction::Check => "Would use",
-                SyncAction::Update => "Would update",
-                SyncAction::Replace => "Would replace",
-                SyncAction::Create => "Would create",
+                Self::Check => "Would use",
+                Self::Update => "Would update",
+                Self::Replace => "Would replace",
+                Self::Create => "Would create",
             }
         } else {
             // For projects, we omit some of these messages when we're not in dry-run mode
             let is_project = matches!(target, TargetName::Project);
             match self {
-                SyncAction::Check | SyncAction::Update | SyncAction::Create if is_project => {
+                Self::Check | Self::Update | Self::Create if is_project => {
                     return None;
                 }
-                SyncAction::Check => "Using",
-                SyncAction::Update => "Updating",
-                SyncAction::Replace => "Replacing",
-                SyncAction::Create => "Creating",
+                Self::Check => "Using",
+                Self::Update => "Updating",
+                Self::Replace => "Replacing",
+                Self::Create => "Creating",
             }
         };
         Some(message)
@@ -1095,10 +1100,10 @@ impl LockAction {
     fn message(&self, dry_run: bool) -> Option<&'static str> {
         let message = if dry_run {
             match self {
-                LockAction::Use => return None,
-                LockAction::Check => "Found up-to-date",
-                LockAction::Update => "Would update",
-                LockAction::Create => "Would create",
+                Self::Use => return None,
+                Self::Check => "Found up-to-date",
+                Self::Update => "Would update",
+                Self::Create => "Would create",
             }
         } else {
             return None;
@@ -1152,7 +1157,7 @@ impl From<&PythonEnvironment> for EnvironmentReport {
 
 impl From<&SyncEnvironment> for EnvironmentReport {
     fn from(env: &SyncEnvironment) -> Self {
-        let report = EnvironmentReport::from(&**env);
+        let report = Self::from(&**env);
         // Replace the path if necessary; we construct a temporary virtual environment during dry
         // run invocations and want to report the path we _would_ use.
         if let Some(path) = env.dry_run_target() {
