@@ -5,10 +5,8 @@ use anyhow::Result;
 use assert_cmd::assert::OutputAssertExt;
 use assert_fs::fixture::{FileWriteStr, PathChild, PathCreateDir};
 use insta::assert_snapshot;
-use uv_python::{
-    PYTHON_VERSION_FILENAME, PYTHON_VERSIONS_FILENAME,
-    platform::{Arch, Os},
-};
+use uv_platform::{Arch, Os};
+use uv_python::{PYTHON_VERSION_FILENAME, PYTHON_VERSIONS_FILENAME};
 
 #[test]
 fn python_pin() {
@@ -847,7 +845,7 @@ fn python_pin_rm() {
     error: No Python version file found
     ");
 
-    // Remove the local pin
+    // Create and remove a local pin
     context.python_pin().arg("3.12").assert().success();
     uv_snapshot!(context.filters(), context.python_pin().arg("--rm"), @r"
     success: true
@@ -884,12 +882,41 @@ fn python_pin_rm() {
         .arg("--global")
         .assert()
         .success();
+
     uv_snapshot!(context.filters(), context.python_pin().arg("--rm").arg("--global"), @r"
     success: true
     exit_code: 0
     ----- stdout -----
-    Removed Python version file at `[UV_USER_CONFIG_DIR]/.python-version`
+    Removed global Python pin at `[UV_USER_CONFIG_DIR]/.python-version`
 
     ----- stderr -----
     ");
+
+    // Add the global pin again
+    context
+        .python_pin()
+        .arg("3.12")
+        .arg("--global")
+        .assert()
+        .success();
+
+    // Remove the local pin
+    uv_snapshot!(context.filters(), context.python_pin().arg("--rm"), @r"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    Removed Python version file at `.python-version`
+
+    ----- stderr -----
+    ");
+
+    // The global pin should not be removed without `--global`
+    uv_snapshot!(context.filters(), context.python_pin().arg("--rm"), @r"
+    success: false
+    exit_code: 2
+    ----- stdout -----
+
+    ----- stderr -----
+    error: No Python version file found; use `--rm --global` to remove the global pin
+    ");
 }
